<?php
######################################################################################################################
# mix_keywords(mixing type, url template, root keywords array, branch keywords array, optional coutner)
# mixes two sets of keywords based on the options set
######################################################################################################################
function mix_keywords($mixing, $url, $rootKeywords, $branchKeywords, $x=0) {
	global $rootBase;
	global $branchBase;
	global $urlTemplate;
	global $adGroups;

	# loop through each root keyword
	foreach ($rootKeywords as $root) {
		# ignore the branch keywords
		if ($mixing == 'nomix') {
			combine_words('nomix', null, $url, $root); 
		}
		# pair with the branch keywords
		elseif ($mixing == 'pair') {
			combine_words('pair', null, $url, $root, $branchKeywords[$x]);
			$x++;
		}
		# mix root and branch keywords together
		else {
			foreach ($branchKeywords as $branch) {
				if ($mixing == 'before' || $mixing == 'both') { combine_words('before', null, $url, $root, $branch); }
				if ($mixing == 'after' || $mixing == 'both') { combine_words('after', null, $url, $root, $branch); }
			}
		}
	}
	# loop through each ad group to add the base keywords
	foreach ($adGroups AS $adGroup) {
		if ($rootBase == 'yes') { combine_words('nomix', $adGroup['title'], $url, $adGroup['root']); }
		if ($branchBase == 'yes') { combine_words('nomix', $adGroup['title'], $url, $adGroup['branch']); }
	}
}

######################################################################################################################
# combine_words(combining mode, url template, root keyword, optional branch keyword)
# takes two keywords, combines them, does some work on them, and forms a CSV record
######################################################################################################################
function combine_words($mode, $group, $url, $root, $branch=null) {
	# access preset variables
	global $campaignName;
	global $matchTypes;
	global $bidAmounts;
	global $actualSeparator;
	global $csvOutput;
	
	# get the keyword group name
	if ($group == null) { $group = get_current_group($root, $branch); }

	# how should we mix the keywords
 	if ($mode == 'before') { $keyword = $branch.' '.$root; }
	if ($mode == 'after') { $keyword = $root.' '.$branch; }
	if ($mode == 'pair' || $mode == 'nomix' || $mode == 'base') { $keyword = $root; }

	# replace spaces with chosen separator in the keyword url
	$urlTokens = array(
		'root'		=> $root,
		'branch'	=> $branch,
		'mix'		=> $keyword,
		'group'		=> $group,
	);
	$urlTokens = str_replace(' ', $actualSeparator, $urlTokens);

	# replace tokens in the keyword url
	$find_replace = array(
		'{root}'	=> $urlTokens['root'],
		'{Root}'	=> $urlTokens['root'],
		'{branch}'	=> $urlTokens['branch'],
		'{Branch}'	=> $urlTokens['branch'],
		'{mix}'		=> $urlTokens['mix'],
		'{Mix}'		=> $urlTokens['mix'],
		'{group}'	=> $urlTokens['group'],
		'{Group}'	=> $urlTokens['group'],		
	);
	$url = str_replace(array_keys($find_replace), array_values($find_replace), $url);
	
	# build a unique record of this variable for each match type
	foreach ($matchTypes as $match) {
		# replace matchtype token in url
		$finalUrl = strtolower(str_replace(array('{match}','{Match}'), $match, $url));
		# build the csv records
		$csvOutput['adw']['kw'] .= '"'.$campaignName.'","'.$group.'","'.strtolower($keyword).'","'.$match.'","'.$bidAmounts[$match].'","'.$finalUrl.'"'."\n";
	}
}

######################################################################################################################
# get_current_group(root keyword, optional branch keyword)
# determines and returns the proper group name based on two keywords and global variables
######################################################################################################################
function get_current_group($root, $branch=null) {
	# define global variables
	global $mixing;
	global $grouping;
	global $groupCapital;
	global $groupSeparator;
	global $groupName;
	global $extra;
	global $extraText;
	global $adGroups;
	
	# create the basic ad group name
	if ($grouping == 'root') { $group = $root; }
	elseif ($grouping == 'branch') { $group = $branch; }
	elseif ($grouping == 'single') { $group = $groupName; }
	elseif ($grouping == 'final') { 
		if ($mixing == 'after') { $group = $root.' '.$branch; }
		else { $group = $branch.' '.$root; }
	}
	
	# format the ad group name
	if ($groupCapital == 'upper') { $group = ucwords($group); }
	elseif ($groupCapital == 'lower') { $group = strtolower($group); }
	
	if ($groupSeparator != null) { 
		if ($groupSeparator == 'blank') { $group = str_replace(' ', '', $group); }
		else { $group = str_replace(' ', $groupSeparator, $group); }
	}
	
	# define and append a prefix or suffix
	if ($extra == 'prefix') { $prefix = $extraText; }
	if ($extra == 'suffix') { $suffix = $extraText; }
	$group = $prefix.$group.$suffix;

	# if new, add to our list of groups
	if (!isset($adGroups[$group])) {
		$adGroups[$group]['title'] = $group;
		$adGroups[$group]['root'] = $root;
		$adGroups[$group]['branch'] = $branch;
		$adGroups[$group]['mix'] = $branch.' '.$root;
	}
	
	# return the final group name
	return $group;
}

######################################################################################################################
# build_ads()
# uses global variables to loop through an ad group array and create text ads
######################################################################################################################
function build_ads() {
	# access preset variables
	global $headline;
	global $line1;
	global $line2;
	global $display;
	global $actual;
	global $adGroups;
	global $campaignName;
	global $displaySeparator;
	global $actualSeparator;
	global $csvOutput;

	# delete empty ads
	foreach($headline as $key => $value) {
		if($value == '') {
			unset($headline[$key]);
		}
	}
	
	# loop through each ad group
	foreach ($adGroups AS $adGroup) {
		# loop through each text ad
		foreach ($headline AS $key => $value) {
			# get the ad components
			$ad['headline'] = $headline[$key];
			$ad['line1'] = $line1[$key];
			$ad['line2'] = $line2[$key];
			$ad['displayUrl'] = $display[$key];
			$ad['actualUrl'] = $actual[$key];
			
			# replace tokens in the ad
			$find_replace = array(
				'/{root:?(.*?)}/'		=> strtolower($adGroup['root']),
				'/{Root:?(.*?)}/'		=> ucwords($adGroup['root']),
				'/{branch:?(.*?)}/'		=> strtolower($adGroup['branch']),
				'/{Branch:?(.*?)}/'		=> ucwords($adGroup['branch']),
				'/{mix:?(.*?)}/'		=> strtolower($adGroup['mix']),
				'/{Mix:?(.*?)}/'		=> ucwords($adGroup['mix']),
				'/{group:?(.*?)}/'		=> strtolower($adGroup['title']),
				'/{Group:?(.*?)}/'		=> ucwords($adGroup['title']),	
				'/{campaign:?(.*?)}/'	=> strtolower($campaignName),
				'/{Campaign:?(.*?)}/'	=> ucwords($campaignName),		
			);
			$ad_temp = preg_replace(array_keys($find_replace), array_values($find_replace), $ad);
			
			# if ad lines are too long, insert default text
			$ad['headline'] = check_ad_length($ad['headline'], $ad_temp['headline'], 25);
			$ad['line1'] = check_ad_length($ad['line1'], $ad_temp['line1'], 35);
			$ad['line2'] = check_ad_length($ad['line2'], $ad_temp['line2'], 35);
			$ad['displayUrl'] = check_ad_length($ad['displayUrl'], $ad_temp['displayUrl'], 35);
			
			# reparse any tokens that do not have default text
			$ad = preg_replace(array_keys($find_replace), array_values($find_replace), $ad);
	
			# replace spaces
			$ad['displayUrl'] = str_replace(' ', $displaySeparator, $ad['displayUrl']);
			$ad['actualUrl'] = str_replace(' ', $actualSeparator, $ad['actualUrl']);
			
			#build the csv records
			$csvOutput['adw']['ad'] .= '"'.$campaignName.'","'.$adGroup['title'].'","'.$ad['headline'].'","'.$ad['line1'].'","'.$ad['line2'].'","'.$ad['displayUrl'].'","'.$ad['actualUrl'].'"'."\n";	
		} 	
	}
}

######################################################################################################################
# check_ad_length(original string, temporary string, )
# checks temporary text against the limit and replaces original tokens with defaults if it's too long
######################################################################################################################
function check_ad_length($orig, $temp, $limit) {
	# check if this line is beyond the limit
	if (strlen($temp) > $limit) {
		# get tokens and default text
		$pattern = '/{(.*?):(.*?)}/';
		preg_match_all($pattern, $orig, $matches);
		
		# replace tokens that have default text
		$temp = str_replace($matches[0], array_map('trim', $matches[2]), $orig);
	}
	
	#return the result
	return $temp;
}

?>